#
# modeler TCL script
#
# cse 457 spring 1999 project 2
#
# douglas zongker
# dougz@cs.washington.edu
#


# default "animation" menu settings:  animation off.
set g_timebase   0 
set g_currtime   0
set g_timerun    0
set g_oldtimerun 0

# project 4 stuff
#source curve.tcl

# user should have the MODELER environment variable set to the path
# containing the modeler DLL.  If it's not set or set wrong, complain
# and quit.
if {[catch {load a:\\modeler}]} {
    wm withdraw .
    tk_dialog .error "Error" "Failed to load a:\\modeler.dll!" error 0 Quit
    exit
}

# start_time starts the animation if it's not already going.
proc start_time {} {
    global g_oldtimerun g_timerun g_timebase g_currtime

    if {$g_timerun && $g_oldtimerun==0} {
	set g_timebase [clock clicks]
	timed_redraw
    } 
    set g_oldtimerun $g_timerun
}

# timed_redraw is called periodically when animation is on.
# it invalidates the output window, forcing redraw of the model,
# then schedules itself to run again.
proc timed_redraw {} {
    global g_timerun g_timebase g_currtime

    if {$g_timerun} {
	set g_currtime [clock clicks]
	opengl::invalidate
	after [expr 1000/$g_timerun] timed_redraw
    } else {
	set g_timebase 0
	set g_currtime 0
    }
}

# reset_time changes the base time to the current time, which
# effectively resets the animation time to 0.0
proc reset_time {} {
    global g_timerun g_timebase g_currtime
    set g_timebase $g_currtime
}

# control_change is called whenever the value of a control
# widget changes.  It forces redraw of the model.
proc control_change {{args}} {
    opengl::invalidate
}

# load_model is called with the name of a model DLL to open.
proc load_model {dll} {
    # load the binary
    if {[catch [list load $dll]]} {
	tk_dialog .error "Error" "Failed to load model \"$dll\"!" error 0 Ok
	return
    }

    # call the model's setup function
    model::setup

    # create an output window and attach the model's redraw
    # and resize functions to it.
    opengl::create model::redraw model::resize
}

# unload_mode is called to close a model.
proc unload_model {} {
    # stop the animation
    global g_timerun
    set g_timerun 0

    # destroy the output window
    opengl::destroy

    # reset the modeler (which deletes all control widgets)
    model::reset
}

# select_model is called to allow the user to pick a model to
# load.
proc select_model {} {
    # use the default file picker to get a DLL
    set fn [tk_getOpenFile -defaultextension "dll" -filetypes [list {{DLLs} {.dll}}]]

    # if user hit cancel, return
    if {[string compare $fn ""]==0} return

    # unload the current model, if any
    close_model

    # load the new model
    load_model $fn

    # force redraw
    opengl::invalidate
}

# close_model is called to remove the current model (either by
# user request, or when a new model is about to be loaded)
proc close_model {} {
    # project 4 stuff
    #close_curve_editor
    #clear_curves

    # unload the model
    unload_model
}

proc write_raytrace {} {
    global g_drawquality g_drawmode g_rayfilename
    set fn [tk_getSaveFile -defaultextension "ray" -filetypes [list {{Ray tracer input files} {.ray}}]]
    if {[string compare $fn ""]==0} return

    set g_rayfilename $fn
    set g_drawmode [expr 8|$g_drawquality]
    opengl::invalidate
    update
    

    set_drawmode
}

# set_drawmode is called when something on the view menu is
# selected.  It updates the g_drawmode variable with the
# new bit flags.
proc set_drawmode {{args}} {
    global g_drawmode g_drawtype g_drawquality
    
    # the drawing MODE is the drawing TYPE (normal, wireframe, flatshade)
    # flag OR'd together with the QUALITY (high, medium, low)
    set g_drawmode [expr $g_drawtype|$g_drawquality]

    # redraw the model
    opengl::invalidate
}

# build_menubar is called, not surprisingly, to set up the main window's
# menu bar.
proc build_menubar {} {
    global g_drawtype g_drawquality
    
    # create the root menu
    menu .mb -tearoff false

    # file menu
    menu .mb.file -tearoff false
    .mb.file add command -label "Open model..." -underline 0 -command select_model
    .mb.file add command -label "Close model" -underline 0 -command close_model
    .mb.file add separator
    .mb.file add command -label "Write raytrace file..." -underline 7 -command write_raytrace
    .mb.file add separator
    .mb.file add command -label "Quit" -underline 0 -command exit
    .mb add cascade -label "File" -underline 0 -menu .mb.file

    # edit menu
    menu .mb.edit -tearoff false
    .mb.edit add command -label "Redraw" -underline 0 -command opengl::invalidate
    #.mb.edit add command -label "Curve editor..." -underline 0 -command open_curve_editor
    .mb add cascade -label "Edit" -underline 0 -menu .mb.edit

    # view menu
    menu .mb.view -tearoff false
    .mb.view add radiobutton -label "Normal" -underline 0 -variable g_drawtype -value 1
    .mb.view add radiobutton -label "Wireframe" -underline 0 -variable g_drawtype -value 2
    .mb.view add radiobutton -label "Flat shaded" -underline 0 -variable g_drawtype -value 4
    .mb.view add separator
    .mb.view add radiobutton -label "High quality" -underline 0 -variable g_drawquality -value 256
    .mb.view add radiobutton -label "Medium quality" -underline 0 -variable g_drawquality -value 512
    .mb.view add radiobutton -label "Low quality" -underline 0 -variable g_drawquality -value 1024
    .mb add cascade -label "View" -underline 0 -menu .mb.view

    # default "view" menu settings: normal mode, high quality
    set g_drawtype    1      
    set g_drawquality 256    

    # whenever a view menu item is changed, call the set_drawmode function
    trace variable g_drawtype w set_drawmode
    trace variable g_drawquality w set_drawmode

    # also, call it now, with the default values 
    set_drawmode

    # animation menu 
    menu .mb.animate -tearoff false
    .mb.animate add radiobutton -label "Animation off" -variable g_timerun -value 0 -command start_time -underline 0
    .mb.animate add radiobutton -label "5 frames/second" -variable g_timerun -value 5 -command start_time -underline 0
    .mb.animate add radiobutton -label "10 frames/second" -variable g_timerun -value 10 -command start_time -underline 0
    .mb.animate add radiobutton -label "20 frames/second" -variable g_timerun -value 20 -command start_time -underline 0
    .mb.animate add radiobutton -label "30 frames/second" -variable g_timerun -value 30 -command start_time -underline 0
    .mb.animate add separator
    .mb.animate add command -label "Reset time" -underline 0 -command reset_time
    .mb add cascade -label "Animate" -underline 0 -menu .mb.animate
}

###
### here is the main script -- not much to it, is there?
###
### this just sets up the main window; everything else
### is handled through callbacks.
###

# create the menu and put it in the window
build_menubar
. configure -menu .mb

# create the empty control panel and put it in the window
frame .c -width 4i
pack .c

# if user gave a model name on the command line, try to load it.
if {[llength $argv]>0} {
    load_model [lindex $argv 0]
}

